# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for fedmsg.meta """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.compose import *
from fedmsg_meta_fedora_infrastructure.tests.pkgdb import *
from fedmsg_meta_fedora_infrastructure.tests.planet import *
from fedmsg_meta_fedora_infrastructure.tests.buildsys import *
from fedmsg_meta_fedora_infrastructure.tests.askbot import *
from fedmsg_meta_fedora_infrastructure.tests.tagger import *
from fedmsg_meta_fedora_infrastructure.tests.trac import *
from fedmsg_meta_fedora_infrastructure.tests.mailman3 import *
from fedmsg_meta_fedora_infrastructure.tests.badges import *
from fedmsg_meta_fedora_infrastructure.tests.ansible import *
from fedmsg_meta_fedora_infrastructure.tests.scm import *
from fedmsg_meta_fedora_infrastructure.tests.datanommer import *
from fedmsg_meta_fedora_infrastructure.tests.nuancier import *
from fedmsg_meta_fedora_infrastructure.tests.fedocal import *
from fedmsg_meta_fedora_infrastructure.tests.coprs import *
from fedmsg_meta_fedora_infrastructure.tests.cnucnuweb import *
from fedmsg_meta_fedora_infrastructure.tests.summershum import *
from fedmsg_meta_fedora_infrastructure.tests.jenkins import *
from fedmsg_meta_fedora_infrastructure.tests.github import *
from fedmsg_meta_fedora_infrastructure.tests.ftpsync import *
from fedmsg_meta_fedora_infrastructure.tests.bz import *
from fedmsg_meta_fedora_infrastructure.tests.elections import *
from fedmsg_meta_fedora_infrastructure.tests.fmn import *

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from fedmsg_meta_fedora_infrastructure.tests.common import add_doc


class TestFASUserCreateLegacy(Base):
    """ Support old fas messages (the new ones look different).

    :mod:`fedmsg.meta` needs to be able to handle these since they are stored
    *forever* in datanommer.
    """
    expected_title = "fas.user.create"
    expected_subti = "New FAS account:  'ralph'  (created by 'ralph')"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph'])
    msg = {
        u'i': 1,
        u'timestamp': 1344432054.8098609,
        u'topic': u'org.fedoraproject.stg.fas.user.create',
        u'msg': {
            u'user': {
                u'username': u'ralph'
            },
            u'agent': {
                u'username': u'ralph'
            }
        }
    }


class TestFASUserCreate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a new user account is created.
    """
    expected_title = "fas.user.create"
    expected_subti = "New FAS account:  'ralph'  (created by 'ralph')"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph'])
    msg = {
        u'i': 1,
        u'timestamp': 1344432054.8098609,
        u'topic': u'org.fedoraproject.stg.fas.user.create',
        u'msg': {
            u'user': u'ralph',
            u'agent': u'ralph',
        }
    }


class TestFASEditProfile(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's account is modified.
    Information about which account, what fields changed, and who did the
    changing are included in the message body.  For example:
    """
    expected_title = "fas.user.update"
    expected_subti = "ralph edited the following fields of ralph's " + \
        "FAS profile:  comments"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.user.update',
        u'msg': {
            u'fields': [u'comments'],
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'ralph'},
        }
    }


class TestFASEditGroupLegacy(Base):
    """ Support old fas messages (the new ones look different).

    :mod:`fedmsg.meta` needs to be able to handle these since they are stored
    *forever* in datanommer.
    """
    expected_title = "fas.group.update"
    expected_subti = "ralph edited the following fields of the " + \
        "ambassadors FAS group:  display_name"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.update',
        u'msg': {
            u'fields': [u'display_name'],
            u'group': {u'name': u'ambassadors'},
            u'agent': {u'username': u'ralph'},
        }
    }


class TestFASEditGroup(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a group's properties are
    modified.  For example:
    """
    expected_title = "fas.group.update"
    expected_subti = "ralph edited the following fields of the " + \
        "ambassadors FAS group:  display_name"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.update',
        u'msg': {
            u'fields': [u'display_name'],
            u'group': u'ambassadors',
            u'agent': u'ralph',
        }
    }


class TestFASGroupCreate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a new group is created.
    """
    expected_title = "fas.group.create"
    expected_subti = "ralph created new FAS group ambassadors"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.create',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'agent': {u'username': u'ralph'},
        }
    }


class TestFASRoleUpdate(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's role in a particular
    group changes.
    """
    expected_title = "fas.role.update"
    expected_subti = "toshio changed ralph's role in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "8128b4c81d09ada7f95ac9dbf888fbea?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.role.update',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'toshio'},
        }
    }


class TestFASGroupRemove(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user is **removed** from a
    particular group.
    """
    expected_title = "fas.group.member.remove"
    expected_subti = "toshio removed ralph from the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "8128b4c81d09ada7f95ac9dbf888fbea?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.remove',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'toshio'},
        }
    }


class TestFASGroupSponsor(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user's request to join a
    restricted group is **sponsored** by an authorized user.
    """
    expected_title = "fas.group.member.sponsor"
    expected_subti = "toshio sponsored ralph's membership " + \
        "in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "8128b4c81d09ada7f95ac9dbf888fbea?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph', 'toshio'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.sponsor',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'toshio'},
        }
    }


class TestFASGroupApply(Base):
    """ The `Fedora Account System <https://admin.fedoraproject.org/accounts>`_
    publishes messages on this topic whenever a user **requests to join** a
    particular group.
    """
    expected_title = "fas.group.member.apply"
    expected_subti = "ralph applied for ralph's membership " + \
        "in the ambassadors group"
    expected_icon = "https://admin.fedoraproject.org/accounts/static" + \
        "/theme/fas/images/account.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['users/ralph', 'groups/ambassadors'])
    msg = {
        u'topic': u'org.fedoraproject.stg.fas.group.member.apply',
        u'msg': {
            u'group': {u'name': u'ambassadors'},
            u'user': {u'username': u'ralph'},
            u'agent': {u'username': u'ralph'},
        }
    }


class LegacyTestBodhiUpdateComplete(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever an update
    **completes it's push to the testing repository**.  Here's a
    straightforward example:

    .. note:: We *used* to have this message in our system, but it got removed
       when we found that fedmsg caused the bodhi1 masher to segfault in
       certain circumstances.  This message, or one like it will likely return
       with the advent of bodhi2.
    """
    expected_title = "bodhi.update.complete.testing"
    expected_subti = "ralph's fedmsg-0.2.7-2.el6 bodhi update " + \
        "completed push to testing"
    expected_link = "https://admin.fedoraproject.org/updates/" + \
        "fedmsg-0.2.7-2.el6"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_packages = set(['fedmsg'])
    expected_objects = set(['packages/fedmsg'])

    msg = {
        "i": 88,
        "timestamp": 1344447839.891876,
        "topic": "org.fedoraproject.prod.bodhi.update.complete.testing",
        "msg": {
            "update": {
                "close_bugs": True,
                "critpath": False,
                "stable_karma": 3,
                "date_pushed": 1344447839.0,
                "title": "fedmsg-0.2.7-2.el6",
                "nagged": None,
                "comments": [
                    {
                        "group": None,
                        "author": "bodhi",
                        "text": "This update has been submitted for " +
                        "testing by ralph. ",
                        "karma": 0,
                        "anonymous": False,
                        "timestamp": 1344266157.0
                    },
                    {
                        "group": None,
                        "author": "bodhi",
                        "text": "This update is currently being pushed " +
                        "to the Fedora EPEL 6 testing updates " +
                        "repository.",
                        "karma": 0,
                        "anonymous": False,
                        "timestamp": 1344443927.0
                    }
                ],
                "updateid": "FEDORA-EPEL-2012-6650",
                "type": "bugfix",
                "status": "testing",
                "date_submitted": 1344266152.0,
                "unstable_karma": -3,
                "release": {
                    "dist_tag": "dist-6E-epel",
                    "id_prefix": "FEDORA-EPEL",
                    "locked": False,
                    "name": "EL-6",
                    "long_name": "Fedora EPEL 6"
                },
                "approved": None,
                "builds": [
                    {
                        "nvr": "fedmsg-0.2.7-2.el6",
                        "package": {
                            "suggest_reboot": False,
                            "committers": [
                                "ralph"
                            ],
                            "name": "fedmsg"
                        }
                    }
                ],
                "date_modified": None,
                "notes": "Bugfix - Added a forgotten new " +
                "requirement on python-requests.",
                "request": None,
                "bugs": [],
                "critpath_approved": False,
                "karma": 0,
                "submitter": "ralph",
            }
        }
    }


class TestBodhiRequestMultiplePackagesPerUpdate(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a *user* requests that an update
    be pushed to the testing repository. Some updates may contain *multiple
    packages*, which can be a little tricky if you're not ready for it.  Here's
    an example of that:
    """
    expected_title = "bodhi.update.request.testing"
    expected_subti = "lmacken submitted " + \
        "gnome-settings-daemon-3.6.1-1.fc18,control-center-3.6.1-1.fc18" + \
        " to testing"
    expected_link = "https://admin.fedoraproject.org/updates/" + \
        "gnome-settings-daemon-3.6.1-1.fc18,control-center-3.6.1-1.fc18"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "9d953fa825bd80dfa6e45660b03adc2d?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['hadess'])
    expected_packages = set(['gnome-settings-daemon', 'control-center'])
    expected_objects = set([
        'packages/gnome-settings-daemon',
        'packages/control-center',
    ])

    msg = {
        "topic": "org.fedoraproject.prod.bodhi.update.request.testing",
        "msg": {
            'agent': 'lmacken',
            "update": {
                "status": "pending",
                "critpath": False,
                "stable_karma": 3,
                "date_pushed": None,
                "title": "gnome-settings-daemon-3.6.1-1.fc18," +
                "control-center-3.6.1-1.fc18",
                "nagged": None,
                "comments": [
                    {
                        "group": None,
                        "author": "bodhi",
                        "text": "This update has been submitted for "
                        "testing by hadess. ",
                        "karma": 0,
                        "anonymous": False,
                        "timestamp": 1349718539.0,
                        "update_title": "gnome-settings-daemon-3.6.1-1.fc18," +
                        "control-center-3.6.1-1.fc18"
                    }
                ],
                "updateid": None,
                "type": "bugfix",
                "close_bugs": True,
                "date_submitted": 1349718534.0,
                "unstable_karma": -3,
                "release": {
                    "dist_tag": "f18",
                    "locked": True,
                    "long_name": "Fedora 18",
                    "name": "F18",
                    "id_prefix": "FEDORA"
                },
                "approved": None,
                "builds": [
                    {
                        "nvr": "gnome-settings-daemon-3.6.1-1.fc18",
                        "package": {
                            "suggest_reboot": False,
                            "committers": [
                                "hadess",
                                "ofourdan",
                                "mkasik",
                                "cosimoc"
                            ],
                            "name": "gnome-settings-daemon"
                        }
                    }, {
                        "nvr": "control-center-3.6.1-1.fc18",
                        "package": {
                            "suggest_reboot": False,
                            "committers": [
                                "ctrl-center-team",
                                "ofourdan",
                                "ssp",
                                "ajax",
                                "alexl",
                                "jrb",
                                "mbarnes",
                                "caolanm",
                                "davidz",
                                "mclasen",
                                "rhughes",
                                "hadess",
                                "johnp",
                                "caillon",
                                "whot",
                                "rstrode"
                            ],
                            "name": "control-center"
                        }
                    }
                ],
                "date_modified": None,
                "notes": "This update fixes numerous bugs in the new Input " +
                "Sources support, the Network panel and adds a help " +
                "screen accessible via Wacom tablets's buttons.",
                "request": "testing",
                "bugs": [],
                "critpath_approved": False,
                "karma": 0,
                "submitter": "hadess"
            }
        },
        "i": 2,
        "timestamp": 1349718539.0,
    }


class TestBodhiMashTaskMashing(Base):
    """ The `Bodhi Masher <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever it starts mashing
    a particular repository.

    Note that, these messages are broken (serverside) due to `this
    issue <https://github.com/fedora-infra/fedmsg/issues/115>`_.
    """
    expected_title = "bodhi.mashtask.mashing"
    expected_subti = "bodhi masher is mashing test_repo"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = ''
    expected_objects = set(['repos/test_repo'])

    msg = {
        'topic': "org.fedoraproject.prod.bodhi.mashtask.mashing",
        'msg': {
            'repo': 'test_repo',
        },
    }


class TestBodhiMashTaskStart(Base):
    """ The `Bodhi Masher <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever it **begins** its work.

    Note that, these messages are broken (serverside) due to `this
    issue <https://github.com/fedora-infra/fedmsg/issues/115>`_.
    """
    expected_title = "bodhi.mashtask.start"
    expected_subti = "bodhi masher started its mashtask"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = ''
    msg = {
        'topic': "org.fedoraproject.prod.bodhi.mashtask.start",
        'msg': {}
    }


class TestBodhiMashTaskComplete(Base):
    """ The `Bodhi Masher <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever it **finishes** its work.

    Note that, these messages are broken (serverside) due to `this
    issue <https://github.com/fedora-infra/fedmsg/issues/115>`_.
    """
    expected_title = "bodhi.mashtask.complete"
    expected_subti = "bodhi masher failed to complete its mashtask!"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = ''
    msg = {
        'topic': "org.fedoraproject.prod.bodhi.mashtask.complete",
        'msg': {'success': False}
    }


class TestBodhiMashTaskSyncWaitStart(Base):
    """ The `Bodhi Masher <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic when it begins **waiting to sync**.

    Note that, these messages are broken (serverside) due to `this
    issue <https://github.com/fedora-infra/fedmsg/issues/115>`_.
    """
    expected_title = "bodhi.mashtask.sync.wait"
    expected_subti = "bodhi masher is waiting on mirror repos to sync"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = ''
    msg = {
        'topic': "org.fedoraproject.prod.bodhi.mashtask.sync.wait",
        'msg': {}
    }


class TestBodhiMashTaskSyncWaitDone(Base):
    """ The `Bodhi Masher <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic when it finishes syncing.

    Note that, these messages are broken (serverside) due to `this
    issue <https://github.com/fedora-infra/fedmsg/issues/115>`_.
    """
    expected_title = "bodhi.mashtask.sync.done"
    expected_subti = "bodhi masher finished waiting on mirror repos to sync"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = ''

    msg = {
        'topic': "org.fedoraproject.prod.bodhi.mashtask.sync.done",
        'msg': {}
    }


class TestBodhiRequestUnpush(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a *user* requests that an update
    be **unpushed**.
    """
    expected_title = "bodhi.update.request.unpush"
    expected_subti = "lmacken unpushed foo"
    expected_link = "https://admin.fedoraproject.org/updates/foo"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['foo'])
    expected_objects = set(['packages/foo'])

    msg = {
        'topic': "org.fedoraproject.dev.bodhi.update.request.unpush",
        'msg': {
            'agent': 'lmacken',
            'update': {
                'title': 'foo',
                'submitter': 'lmacken',
            },
        },
    }


class TestBodhiRequestObsolete(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a *user* requests that an update
    be **obsoleted**.
    """
    expected_title = "bodhi.update.request.obsolete"
    expected_subti = "lmacken obsoleted foo"
    expected_link = "https://admin.fedoraproject.org/updates/foo"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['foo'])
    expected_objects = set(['packages/foo'])

    msg = {
        'topic': "org.fedoraproject.dev.bodhi.update.request.obsolete",
        'msg': {
            'agent': 'lmacken',
            'update': {
                'title': 'foo',
                'submitter': 'lmacken',
            },
        },
    }


class TestBodhiRequestStable(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a *user* requests that an update
    be marked as **stable**.
    """
    expected_title = "bodhi.update.request.stable"
    expected_subti = "lmacken submitted foo to stable"
    expected_link = "https://admin.fedoraproject.org/updates/foo"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['foo'])
    expected_objects = set(['packages/foo'])

    msg = {
        'topic': "org.fedoraproject.dev.bodhi.update.request.stable",
        'msg': {
            'agent': 'lmacken',
            'update': {
                'title': 'foo',
                'submitter': 'lmacken',
            },
        },
    }


class TestBodhiRequestRevoke(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a *user* revokes a prior
    request on an update.
    """
    expected_title = "bodhi.update.request.revoke"
    expected_subti = "lmacken revoked foo"
    expected_link = "https://admin.fedoraproject.org/updates/foo"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['foo'])
    expected_objects = set(['packages/foo'])

    msg = {
        'topic': "org.fedoraproject.dev.bodhi.update.request.revoke",
        'msg': {
            'agent': 'lmacken',
            'update': {
                'title': 'foo',
                'submitter': 'lmacken',
            },
        },
    }


class TestBodhiRequestTesting(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a *user* requests that an
    update be pushed to the testing repository.
    """
    expected_title = "bodhi.update.request.testing"
    expected_subti = "lmacken submitted foo to testing"
    expected_link = "https://admin.fedoraproject.org/updates/foo"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['foo'])
    expected_objects = set(['packages/foo'])

    msg = {
        'topic': "org.fedoraproject.dev.bodhi.update.request.testing",
        'msg': {
            'agent': 'lmacken',
            'update': {
                'title': 'foo',
                'submitter': 'lmacken',
            },
        },
    }


class TestBodhiComment(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a user **comments** on a bodhi
    update.
    """
    expected_title = "bodhi.update.comment"
    expected_subti = "ralph commented on bodhi update fedmsg-1.0-1 (karma: -1)"
    expected_link = "https://admin.fedoraproject.org/updates/fedmsg-1.0-1"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_packages = set(['fedmsg'])
    expected_objects = set(['packages/fedmsg'])

    msg = {
        "i": 1,
        "timestamp": 1344344053.2337201,
        "topic": "org.fedoraproject.stg.bodhi.update.comment",
        "msg": {
            "comment": {
                "update_title": "fedmsg-1.0-1",
                "group": None,
                "author": "ralph",
                "text": "Can you believe how much testing we're doing?",
                "karma": -1,
                "anonymous": False,
                "timestamp": 1344344050.0
            }
        }
    }


class TestBodhiOverrideTagged(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a user **requests a buildroot
    override** for an update.
    """
    expected_title = "bodhi.buildroot_override.tag"
    expected_subti = "lmacken submitted a buildroot override for fedmsg-1.0-1"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['fedmsg'])
    expected_objects = set(['packages/fedmsg'])

    msg = {
        "i": 1,
        "timestamp": 1344344053.2337201,
        "topic": "org.fedoraproject.stg.bodhi.buildroot_override.tag",
        "msg": {
            "override": {
                "build": "fedmsg-1.0-1",
                "submitter": "lmacken",
            }
        }
    }


class TestBodhiOverrideUntagged(Base):
    """ The `Bodhi Updates System <https://admin.fedoraproject.org/updates>`_
    publishes messages on this topic whenever a user explicitly removes a
    previously requested buildroot override.
    """
    expected_title = "bodhi.buildroot_override.untag"
    expected_subti = "lmacken expired a buildroot override for fedmsg-1.0-1"
    expected_icon = "https://admin.fedoraproject.org/updates" + \
        "/static/images/bodhi-icon-48.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "0c35a75019e58e54fb58202db20d2c24?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['lmacken'])
    expected_packages = set(['fedmsg'])
    expected_objects = set(['packages/fedmsg'])

    msg = {
        "i": 1,
        "timestamp": 1344964395.207541,
        "topic": "org.fedoraproject.stg.bodhi.buildroot_override.untag",
        "msg": {
            "override": {
                "build": "fedmsg-1.0-1",
                "submitter": "lmacken",
            }
        }
    }


class TestSupybotStartMeetingNoName(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published (somewhat obviously)
    when a new IRC meeting is started.  The user starting the meeting may
    specify a meeting title, but doesn't have to.  Here's an example
    message with no meeting title specified:
    """
    expected_title = "meetbot.meeting.start"
    expected_subti = 'ralph started a meeting in #channel'
    expected_icon = 'https://apps.fedoraproject.org/img/icons/meetbot.png'
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": None,
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.start",
        "timestamp": 1345572862.556145
    }


class TestSupybotStartMeeting(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published (somewhat obviously)
    when a new IRC meeting is started.  The user starting the meeting may
    specify a meeting title, but doesn't have to.  Here's an example
    message with a specified meeting title:
    """
    expected_title = "meetbot.meeting.start"
    expected_subti = 'ralph started meeting "title" in #channel'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'titles/title',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": "title",
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.start",
        "timestamp": 1345572862.556145
    }


class TestSupybotEndMeeting(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published when an IRC meeting
    ends.  Meetings may or may not have a title (which can be tricky).
    Here's an example message where the title is specified:
    """
    expected_title = "meetbot.meeting.complete"
    expected_subti = 'ralph\'s meeting titled "title" ended in #channel'
    expected_link = 'https://logs.com/awesome.html'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'titles/title',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": "title",
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.complete",
        "timestamp": 1345572862.556145
    }


class TestSupybotEndMeetingNoTitle(Base):
    """ Trusty old `zodbot <https://meetbot.fedoraproject.org/>`_ publishes
    messages too!  Messages on this topic get published when an IRC meeting
    ends.  Meetings may or may not have a title (which can be tricky).
    Here's an example message where the title is **not** specified:
    """
    expected_title = "meetbot.meeting.complete"
    expected_subti = 'ralph\'s meeting ended in #channel'
    expected_link = 'https://logs.com/awesome.html'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": None,
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel"
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.complete",
        "timestamp": 1345572862.556145
    }


class TestSupybotChangeTopic(Base):
    """ As IRC meetings chug along, the chairperson may change the meeting;
    zodbot publishes message for that!  An example **with** a title specified:
    """
    expected_title = "meetbot.meeting.topic.update"
    expected_subti = 'The topic of ralph\'s "title" meeting changed ' +\
        'to "Food" in #channel'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'titles/title',
        'topics/Food',
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": "title",
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel",
            "topic": "Food",
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.topic.update",
        "timestamp": 1345572862.556145
    }


class TestSupybotChangeTopicNoTitle(Base):
    """ As IRC meetings chug along, the chairperson may change the meeting;
    zodbot publishes message for that!  An example **without** a title
    specified:
    """
    expected_title = "meetbot.meeting.topic.update"
    expected_subti = 'The topic of ralph\'s meeting changed ' +\
        'to "Food" in #channel'
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'attendees/ralph',
        'channels/#channel',
        'topics/Food'
    ])

    msg = {
        "i": 16,
        "msg": {
            "meeting_topic": None,
            "attendees": {
                "zodbot": 2,
                "threebean": 2
            },
            "chairs": {},
            "url": "http://logs.com/awesome",
            "owner": "threebean",
            "channel": "#channel",
            "topic": "Food",
        },
        "topic": "org.fedoraproject.dev.meetbot.meeting.topic.update",
        "timestamp": 1345572862.556145
    }


class TestMediaWikiEdit(Base):
    """ Fedora's `Wiki <https://fedoraproject.org/wiki>`_ has a fedmsg hook
    that publishes messages like this one when a user edits a page.
    """
    expected_title = "wiki.article.edit"
    expected_subti = 'Ralph made a wiki edit to "Messaging SIG"'
    expected_link = "https://this-is-a-link.org"
    expected_icon = "https://fedoraproject.org/w/skins/common/" + \
        "images/mediawiki.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['Messaging SIG-page'])

    msg = {
        "topic": "org.fedoraproject.stg.wiki.article.edit",
        "msg": {
            "watch_this": None,
            "base_rev_id": False,
            "title": "Messaging SIG",
            "minor_edit": 0,
            "text": "The diff goes here...",
            "section_anchor": None,
            "summary": "/* Mission */ ",
            "user": "Ralph",
            "revision": None,
            "url": "http://this-is-a-link.org"
        },
        "timestamp": 1344350200
    }


class TestMediaWikiUpload(Base):
    """ Fedora's `Wiki <https://fedoraproject.org/wiki>`_ hook also publishes
    messages when a user upload some media (like a video or a picture).
    """
    expected_title = "wiki.upload.complete"
    expected_subti = 'Ralph uploaded File:Cat.jpg to the wiki: ' + \
        '"This is a beautiful cat..."'
    expected_icon = "https://fedoraproject.org/w/skins/common/" + \
        "images/mediawiki.png"
    expected_usernames = set(['ralph'])
    expected_objects = set(['w/uploads/d/d1/Cat.jpg'])

    msg = {
        "topic": "org.fedoraproject.stg.wiki.upload.complete",
        "msg": {
            "user_id": 8306,
            "description": "This is a beautiful cat",
            "title": {
                "mCascadeSources": [],
                "mLength": -1,
                "mInterwiki": "",
                "mNotificationTimestamp": [],
                "mCascadeRestriction": None,
                "mRedirect": None,
                "mArticleID": 46586,
                "mTextform": "Cat.jpg",
                "mWatched": None,
                "mDbkeyform": "Cat.jpg",
                "mCascadingRestrictions": [],
                "mDefaultNamespace": 0,
                "mRestrictions": [],
                "mUrlform": "Cat.jpg",
                "mLatestID": False,
                "mBacklinkCache": {},
                "mNamespace": 6,
                "mUserCaseDBKey": "Cat.jpg",
                "mTitleProtection": False,
                "mOldRestrictions": False,
                "mFragment": "",
                "mHasCascadingRestrictions": None,
                "mPrefixedText": "File:Cat.jpg",
                "mRestrictionsExpiry": {
                    "create": "infinity"
                },
                "mRestrictionsLoaded": False
            },
            "user_text": "Ralph",
            "minor_mime": "jpeg",
            "url": "/w/uploads/d/d1/Cat.jpg",
            "file_exists": True,
            "mime": "image/jpeg",
            "major_mime": "image",
            "media_type": "BITMAP",
            "size": 295667
        },
        "timestamp": 1344361406
    }


class TestPkgdb2BrMassStart(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Messages on this topic are
    emitted from that script when it is instructed to carry out a "mass
    branch" of all packages.
    """
    expected_title = "git.mass_branch.start"
    expected_subti = "dgilmore started a mass branch"
    expected_usernames = set(['dgilmore'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.mass_branch.start",
        "msg": {
            "agent": "dgilmore",
        },
    }


class TestPkgdb2BrMassComplete(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Messages on this topic are
    emitted from that script when it **finishes** a "mass branch".
    """
    expected_title = "git.mass_branch.complete"
    expected_subti = "mass branch started by dgilmore completed"
    expected_usernames = set(['dgilmore'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.mass_branch.complete",
        "msg": {
            "agent": "dgilmore",
        },
    }


class TestPkgdb2BrRunStart(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **begins**.
    """
    expected_title = "git.pkgdb2branch.start"
    expected_subti = "limburgher started a run of pkgdb2branch"
    expected_usernames = set(['limburgher'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.start",
        "msg": {
            "agent": "limburgher",
        },
    }


class TestPkgdb2BrRunComplete(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **completes**.
    """
    expected_title = "git.pkgdb2branch.complete"
    expected_subti = "run of pkgdb2branch started by limburgher completed"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['nethack'])
    expected_objects = set(['nethack/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.complete",
        "msg": {
            "agent": "limburgher",
            "unbranchedPackages": [],
            "branchedPackages": ["nethack"],
        },
    }


class TestPkgdb2BrRunCompleteWithError(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **completes**.

    *Sometimes* that process can produce errors.  Here's an example of a
    message from a failed ``pkgdb2branch`` run.
    """
    expected_title = "git.pkgdb2branch.complete"
    expected_subti = "run of pkgdb2branch started by limburgher completed" + \
        " with 1 error"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['foo'])
    expected_objects = set(['foo/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.complete",
        "msg": {
            "agent": "limburgher",
            "unbranchedPackages": ['foo'],
            "branchedPackages": [],
        },
    }


class TestPkgdb2BrRunCompleteWithErrors(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published when that process **completes**.

    *Sometimes* that process can produce errors.  Here's an example of a
    message from a failed ``pkgdb2branch`` run (on multiple packages)
    """
    expected_title = "git.pkgdb2branch.complete"
    expected_subti = "run of pkgdb2branch started by limburgher completed" + \
        " with 2 errors"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['foo', 'bar'])
    expected_objects = set(['foo/__git__', 'bar/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.pkgdb2branch.complete",
        "msg": {
            "agent": "limburgher",
            "unbranchedPackages": ['foo', 'bar'],
            "branchedPackages": [],
        },
    }


class TestPkgdb2BrCreateLegacy(Base):
    """ Support old school pkgdb2branch messages.  This don't get published
    anymore, but :mod:`fedmsg.meta` needs to support them since they are
    stored *forever* in datanommer.
    """

    expected_title = "git.branch.valgrind.master"
    expected_subti = \
        "limburgher created branch 'master' for the 'valgrind' package"
    expected_link = \
        "http://pkgs.fedoraproject.org/cgit/valgrind.git/log/?h=master"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.branch.valgrind.master",
        "msg": {
            "agent": "limburgher",
        },
    }


class TestPkgdb2BrCreate(Base):
    """ There is a script called ``pkgdb2branch`` that gets run by an SCM
    admin as part of the new package process.  Typically, when an `SCM Admin
    Request <http://fedoraproject.org/wiki/Package_SCM_admin_requests>`_ is
    approved, the scm admin will add the new package or branch to the package
    database.  *After that*, the scm admin will run ``pkgdb2branch`` to create
    the branch in git on the file system.  Messages of **this** topic are
    published `for each new branch` that that process **creates**.
    """
    expected_title = "git.branch"
    expected_subti = \
        "limburgher created branch 'master' for the 'valgrind' package"
    expected_link = \
        "http://pkgs.fedoraproject.org/cgit/valgrind.git/log/?h=master"
    expected_usernames = set(['limburgher'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/__git__'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.branch",
        "msg": {
            "agent": "limburgher",
            "name": "valgrind",
            "branch": "master",
        },
    }


class TestLookaside(Base):
    """ Messages like this one are published when **new sources** are
    uploaded to the "lookaside cache".
    """

    expected_title = "git.lookaside.new"
    expected_subti = 'jnovy uploaded pst-diffraction.doc.tar.xz for texlive'
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "5b938b3dcb12f46b98a1269cab255cbe?s=64&" + \
        "d=http%3A%2F%2Ffedoraproject.org%2Fstatic%2F" + \
        "images%2Ffedora_infinity_64x64.png"
    expected_link = 'http://pkgs.fedoraproject.org/lookaside/pkgs/' + \
        'texlive/pst-diffraction.doc.tar.xz/' + \
        'dacad985394b3977f9dcf0c75f51a357/' + \
        'pst-diffraction.doc.tar.xz'
    expected_usernames = set(['jnovy'])
    expected_packages = set(['texlive'])
    expected_objects = set(['texlive/pst-diffraction.doc.tar.xz'])

    msg = {
        "i": 1,
        "timestamp": 1349197866.215465,
        "topic": "org.fedoraproject.prod.git.lookaside.new",
        "msg": {
            "agent": "jnovy",
            "md5sum": "dacad985394b3977f9dcf0c75f51a357",
            "name": "texlive",
            "filename": "pst-diffraction.doc.tar.xz"
        }
    }


class TestLookasideLegacy(Base):
    """ Support oldschool lookaside messages.  :( """

    expected_title = "git.lookaside.texlive.new"
    expected_subti = 'jnovy uploaded pst-diffraction.doc.tar.xz for texlive'
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "5b938b3dcb12f46b98a1269cab255cbe?s=64&" + \
        "d=http%3A%2F%2Ffedoraproject.org%2Fstatic%2F" + \
        "images%2Ffedora_infinity_64x64.png"
    expected_link = 'http://pkgs.fedoraproject.org/lookaside/pkgs/' + \
        'texlive/pst-diffraction.doc.tar.xz/' + \
        'dacad985394b3977f9dcf0c75f51a357/' + \
        'pst-diffraction.doc.tar.xz'
    expected_usernames = set(['jnovy'])
    expected_packages = set(['texlive'])
    expected_objects = set(['texlive/pst-diffraction.doc.tar.xz'])

    msg = {
        "i": 1,
        "timestamp": 1349197866.215465,
        "topic": "org.fedoraproject.prod.git.lookaside.texlive.new",
        "msg": {
            "agent": "jnovy",
            "md5sum": "dacad985394b3977f9dcf0c75f51a357",
            "name": "texlive",
            "filename": "pst-diffraction.doc.tar.xz"
        }
    }


class TestSCMSuperLegacy(Base):
    """ Support super-duper oldschool git messages.  :(:( """

    expected_title = "git.receive.valgrind.master"
    expected_subti = 'mjw pushed to valgrind (master).  ' + \
        '"Clear CFLAGS CXXFLAGS LDFLAGS. (..more)"'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "valgrind.git/commit/" + \
        "?h=master&id=7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "923419d315c8f23eface39852bf32a5f?s=64&" + \
        "d=http%3A%2F%2Ffedoraproject.org%2Fstatic%2F" + \
        "images%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['mjw'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/valgrind.spec'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.receive.valgrind.master",
        "msg": {
            "commit": {
                "stats": {
                    "files": {
                        "valgrind.spec": {
                            "deletions": 2,
                            "lines": 3,
                            "insertions": 1
                        }
                    },
                    "total": {
                        "deletions": 2,
                        "files": 1,
                        "insertions": 1,
                        "lines": 3
                    }
                },
                "name": "Mark Wielaard",
                "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
                "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
                "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
                This is a bit of a hammer.""",
                "email": "mjw@redhat.com",
                "username": "mjw",
                "branch": "master",
            }
        }
    }


class TestSCMLegacy(Base):
    """ Support oldschool "fedpkg push" messages.  :( """

    expected_title = "git.receive.valgrind.master"
    expected_subti = 'mjw pushed to valgrind (master).  ' + \
        '"Clear CFLAGS CXXFLAGS LDFLAGS. (..more)"'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "valgrind.git/commit/" + \
        "?h=master&id=7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "923419d315c8f23eface39852bf32a5f?s=64&" + \
        "d=http%3A%2F%2Ffedoraproject.org%2Fstatic%2F" + \
        "images%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['mjw'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/valgrind.spec'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.receive.valgrind.master",
        "msg": {
            "commit": {
                "stats": {
                    "files": {
                        "valgrind.spec": {
                            "deletions": 2,
                            "lines": 3,
                            "insertions": 1
                        }
                    },
                    "total": {
                        "deletions": 2,
                        "files": 1,
                        "insertions": 1,
                        "lines": 3
                    }
                },
                "name": "Mark Wielaard",
                "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
                "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
                "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
                This is a bit of a hammer.""",
                "email": "mjw@redhat.com",
                "username": "mjw",
                "branch": "master",
                "repo": "valgrind",
            }
        }
    }


class TestSCM(Base):
    """ Messages like this one are published when somebody runs "fedpkg push"
    on a package.  Sometimes, the git message may be multiple lines long like:
    """
    expected_title = "git.receive"
    expected_subti = 'mjw pushed to valgrind (master).  ' + \
        '"Clear CFLAGS CXXFLAGS LDFLAGS. (..more)"'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "valgrind.git/commit/" + \
        "?h=master&id=7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "923419d315c8f23eface39852bf32a5f?s=64&" + \
        "d=http%3A%2F%2Ffedoraproject.org%2Fstatic%2F" + \
        "images%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['mjw'])
    expected_packages = set(['valgrind'])
    expected_objects = set(['valgrind/valgrind.spec'])

    msg = {
        "i": 1,
        "timestamp": 1344350850.8867381,
        "topic": "org.fedoraproject.prod.git.receive",
        "msg": {
            "commit": {
                "stats": {
                    "files": {
                        "valgrind.spec": {
                            "deletions": 2,
                            "lines": 3,
                            "insertions": 1
                        }
                    },
                    "total": {
                        "deletions": 2,
                        "files": 1,
                        "insertions": 1,
                        "lines": 3
                    }
                },
                "name": "Mark Wielaard",
                "rev": "7a98f80d9b61ce167e4ef8129c81ed9284ecf4e1",
                "summary": "Clear CFLAGS CXXFLAGS LDFLAGS.",
                "message": """Clear CFLAGS CXXFLAGS LDFLAGS.
                This is a bit of a hammer.""",
                "email": "mjw@redhat.com",
                "username": "mjw",
                "branch": "master",
                "repo": "valgrind",
            }
        }
    }


class TestSCMSingleLine(Base):
    """ Messages like this one are published when somebody runs "fedpkg push"
    on a package.  The whole git message is included for each commit.
    """
    expected_title = "git.receive"
    expected_subti = 'spot pushed to ember (master).  ' + \
        '"another missing patch? ridiculous."'
    expected_link = "http://pkgs.fedoraproject.org/cgit/" + \
        "ember.git/commit/" + \
        "?h=master&id=aa2df80f3d8dd217c7cbfe2d3451190028f3fe14"
    expected_icon = "https://apps.fedoraproject.org/img/icons/git-logo.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "461761d9572bdc1d04925a1125a41797?s=64&" + \
        "d=http%3A%2F%2Ffedoraproject.org%2Fstatic%2F" + \
        "images%2Ffedora_infinity_64x64.png"
    expected_usernames = set(['spot'])
    expected_packages = set(['ember'])
    expected_objects = set(['ember/ember-0.6.3-gcc47.patch'])

    msg = {
        "i": 1,
        "timestamp": 1352998154.368305,
        "topic": "org.fedoraproject.prod.git.receive",
        "msg": {
            "commit": {
                "username": "spot",
                "stats": {
                    "files": {
                        "ember-0.6.3-gcc47.patch": {
                            "deletions": 0,
                            "lines": 26,
                            "insertions": 26
                        }
                    },
                    "total": {
                        "deletions": 0,
                        "files": 1,
                        "insertions": 26,
                        "lines": 26
                    }
                },
                "name": "Tom Callaway",
                "rev": "aa2df80f3d8dd217c7cbfe2d3451190028f3fe14",
                "summary": "another missing patch? ridiculous.",
                "branch": "master",
                "repo": "ember",
                "message": "another missing patch? ridiculous.\n",
                "email": "spot@fedoraproject.org"
            }
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



from fedmsg.tests.test_meta import Base as _Base

import fedmsg_meta_fedora_infrastructure.fasshim


class Base(_Base):
    def setUp(self):
        # We don't want to actually query FAS during our test runs,
        # so mock out _fas_cache to contain a dummy cache.
        fedmsg_meta_fedora_infrastructure.fasshim._fas_cache = {
            'threebean': 'ralph',
            'rbean@redhat.com': 'ralph',
            'puiterwijk@gmail.com': 'puiterwijk',
        }
        super(Base, self).setUp()

    def tearDown(self):
        # At the end of each test, set things back to the way they were.
        fedmsg_meta_fedora_infrastructure.fasshim._fas_cache = {}
        super(Base, self).tearDown()



# This file is part of fedmsg.
# Copyright (C) 2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for bugzilla2fedmsg messages """

import unittest
import datetime

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from common import add_doc


class TestBugzillaUpdate(Base):
    """ Even `Red Hat's Bugzilla instance <https://bugzilla.redhat.com>`_ is
    hooked into fedmsg.

    Messages of *this* type are published whenever someone **updates a bug**
    on the *Fedora* or *Fedora EPEL* products.
    """

    expected_title = "bugzilla.bug.update"
    expected_subti = "ralph updated 'status', 'cc', and 'assigned_to' on " + \
        "RHBZ#968947 'Review Request: gallery3-openid - OpenID...'"
    expected_link = "https://bugzilla.redhat.com/show_bug.cgi?id=968947"
    expected_icon = "https://apps.fedoraproject.org/img/icons/bugzilla.png"
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph', 'puiterwijk'])
    expected_objects = set([
        'Fedora/Package Review/968947',
    ])

    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1395760459,
        "msg_id": "2014-0bc98222-a864-4aea-bc6b-e3b090d2cc3d",
        "topic": "org.fedoraproject.dev.bugzilla.bug.update",
        "msg": {
            "bug": {
                "attachments": [],
                "classification": "Fedora",
                "creator": "puiterwijk@gmail.com",
                "cc": [
                    "package-review@lists.fedoraproject.org",
                    "puiterwijk@gmail.com",
                    "rbean@redhat.com"
                ],
                "depends_on": [],
                "weburl": "https://bugzilla.redhat.com/show_bug.cgi?id=968947",
                "creation_time": 1369924680.0,
                "actual_time": 0.0,
                "docs_contact": "",
                "is_open": True,
                "keywords": [],
                "target_release": [
                    "---"
                ],
                "external_bugs": [],
                "id": 968947,
                "description": "\nSpec URL: http://puiterwijk.fedorapeople..",
                "severity": "unspecified",
                "is_confirmed": True,
                "is_creator_accessible": True,
                "comments": [
                    {
                        "count": 0,
                        "author": "puiterwijk@gmail.com",
                        "text": "\nSpec URL: http://puiterwijk.fedorapeople..",
                        "creator": "puiterwijk@gmail.com",
                        "creation_time": 1369924708.0,
                        "bug_id": 968947,
                        "creator_id": 330125,
                        "time": 1369924708.0,
                        "id": 5962818,
                        "is_private": False
                    },
                    {
                        "count": 1,
                        "author": "puiterwijk@gmail.com",
                        "text": "This package built on koji:    http://koj...",
                        "creator": "puiterwijk@gmail.com",
                        "creation_time": 1369924717.0,
                        "bug_id": 968947,
                        "creator_id": 330125,
                        "time": 1369924717.0,
                        "id": 5962819,
                        "is_private": False
                    },
                    {
                        "count": 2,
                        "author": "rbean@redhat.com",
                        "text": "Some preliminary comments:\n\n* The ...",
                        "creator": "rbean@redhat.com",
                        "creation_time": 1369950294.0,
                        "bug_id": 968947,
                        "creator_id": 269108,
                        "time": 1369950294.0,
                        "id": 5964283,
                        "is_private": False
                    }
                ],
                "priority": "unspecified",
                "platform": "Unspecified",
                "version": "rawhide",
                "fixed_in": "",
                "status": "ASSIGNED",
                "product": "Fedora",
                "blocks": [],
                "qa_contact": "extras-qa@fedoraproject.org",
                "see_also": [],
                "component": "Package Review",
                "remaining_time": 0.0,
                "groups": [],
                "estimated_time": 0.0,
                "target_milestone": "---",
                "is_cc_accessible": True,
                "versions": [
                    "rawhide"
                ],
                "url": "",
                "whiteboard": "",
                "summary": "Review Request: gallery3-openid - OpenID "
                "authentication for Gallery3",
                "alias": [],
                "op_sys": "Unspecified",
                "flags": [
                    {
                        "requestee": "puiterwijk@gmail.com",
                        "status": "?",
                        "name": "needinfo",
                        "modification_date": 1392337756.0,
                        "type_id": 16,
                        "is_active": 1,
                        "creation_date": 1392337756.0,
                        "id": 1703755,
                        "setter": "rbean@redhat.com"
                    }
                ],
                "components": [
                    "Package Review"
                ],
                "assigned_to": "rbean@redhat.com",
                "resolution": "",
                "last_change_time": 1392337756.0
            },
            "event": {
                "changes": [
                    {
                        "removed": "NEW",
                        "field_name": "status",
                        "added": "ASSIGNED"
                    },
                    {
                        "removed": "",
                        "field_name": "cc",
                        "added": "rbean@redhat.com"
                    },
                    {
                        "removed": "nobody@fedoraproject.org",
                        "field_name": "assigned_to",
                        "added": "rbean@redhat.com"
                    }
                ],
                "who": "rbean@redhat.com",
                "when": 1369949333.0
            }
        }
    }


class TestBugzillaNew(Base):
    """ Even `Red Hat's Bugzilla instance <https://bugzilla.redhat.com>`_ is
    hooked into fedmsg.

    Messages of *this* type are published whenever someone **files a new bug**
    on the *Fedora* or *Fedora EPEL* products.

    Note that the ``event`` field is left empty (``{}``) for new bug events.
    """

    expected_title = "bugzilla.bug.new"
    expected_subti = "puiterwijk filed a new bug " + \
        "RHBZ#968947 'Review Request: gallery3-openid - OpenID...'"
    expected_link = "https://bugzilla.redhat.com/show_bug.cgi?id=968947"
    expected_icon = "https://apps.fedoraproject.org/img/icons/bugzilla.png"
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2972a442e97fa492c87f49a3e5fdb5ac?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")

    expected_packages = set([])
    expected_usernames = set(['puiterwijk'])
    expected_objects = set([
        'Fedora/Package Review/968947',
    ])

    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1395760459,
        "msg_id": "2014-0bc98222-a864-4aea-bc6b-e3b090d2cc3d",
        "topic": "org.fedoraproject.dev.bugzilla.bug.new",
        "msg": {
            "bug": {
                "attachments": [],
                "classification": "Fedora",
                "creator": "puiterwijk@gmail.com",
                "cc": [
                    "package-review@lists.fedoraproject.org",
                    "puiterwijk@gmail.com",
                ],
                "depends_on": [],
                "weburl": "https://bugzilla.redhat.com/show_bug.cgi?id=968947",
                "creation_time": 1369924680.0,
                "actual_time": 0.0,
                "docs_contact": "",
                "is_open": True,
                "keywords": [],
                "target_release": [
                    "---"
                ],
                "external_bugs": [],
                "id": 968947,
                "description": "\nSpec URL: http://puiterwijk.fedorapeople..",
                "severity": "unspecified",
                "is_confirmed": True,
                "is_creator_accessible": True,
                "comments": [
                    {
                        "count": 0,
                        "author": "puiterwijk@gmail.com",
                        "text": "\nSpec URL: http://puiterwijk.fedorapeople..",
                        "creator": "puiterwijk@gmail.com",
                        "creation_time": 1369924708.0,
                        "bug_id": 968947,
                        "creator_id": 330125,
                        "time": 1369924708.0,
                        "id": 5962818,
                        "is_private": False
                    },
                ],
                "priority": "unspecified",
                "platform": "Unspecified",
                "version": "rawhide",
                "fixed_in": "",
                "status": "ASSIGNED",
                "product": "Fedora",
                "blocks": [],
                "qa_contact": "extras-qa@fedoraproject.org",
                "see_also": [],
                "component": "Package Review",
                "remaining_time": 0.0,
                "groups": [],
                "estimated_time": 0.0,
                "target_milestone": "---",
                "is_cc_accessible": True,
                "versions": [
                    "rawhide"
                ],
                "url": "",
                "whiteboard": "",
                "summary": "Review Request: gallery3-openid - OpenID "
                "authentication for Gallery3",
                "alias": [],
                "op_sys": "Unspecified",
                "flags": [
                    {
                        "requestee": "puiterwijk@gmail.com",
                        "status": "?",
                        "name": "needinfo",
                        "modification_date": 1392337756.0,
                        "type_id": 16,
                        "is_active": 1,
                        "creation_date": 1392337756.0,
                        "id": 1703755,
                        "setter": "rbean@redhat.com"
                    }
                ],
                "components": [
                    "Package Review"
                ],
                "assigned_to": "",
                "resolution": "",
                "last_change_time": 1392337756.0
            },
            "event": {},
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fasshim import gravatar_url


def repo_name(msg):
    """ Compat util to get the repo name from a message. """
    try:
        # git messages look like this now
        path = msg['msg']['commit']['path']
        project = path.split('.git')[0][9:]
    except KeyError:
        # they used to look like this, though
        project = msg['msg']['commit']['repo']

    return project


class TracProcessor(BaseProcessor):
    __name__ = "Trac"
    __description__ = "events from select Fedora Hosted projects"
    __link__ = "https://fedorahosted.org"
    __docs__ = "https://fedorahosted.org/readme"
    __obj__ = "Fedora Hosted Events"
    __icon__ = "http://www.edgewall.org/gfx/trac_bullet.png"

    def subtitle(self, msg, **config):
        if 'page' in msg['msg']:
            old_name = None
            user = msg['msg']['agent']
            name = msg['msg']['page']['name']
            project = msg['msg']['instance']['project_name']
            print msg['topic']
            if 'page.version.delete' in msg['topic']:
                tmpl = self._(
                    "{user} deleted a version of the '{name}' wiki "
                    "page on the {project} trac instance")
            elif 'page.delete' in msg['topic']:
                tmpl = self._(
                    "{user} straight-up deleted the '{name}' wiki "
                    "page on the {project} trac instance")
            elif 'page.rename' in msg['topic']:
                old_name = msg['msg']['old_name']
                tmpl = self._(
                    "{user} renamed the wiki page '{old_name}' to '{name}' "
                    "on the {project} trac instance")
            elif 'page.update' in msg['topic']:
                tmpl = self._(
                    "{user} updated the '{name}' wiki page "
                    "on the {project} trac instance")
            elif 'page.new' in msg['topic']:
                tmpl = self._(
                    "{user} created a new '{name}' wiki page "
                    "on the {project} trac instance")

            return tmpl.format(
                user=user,
                name=name,
                old_name=old_name,
                project=project)
        elif 'ticket' in msg['msg']:
            status = None
            resolution = None
            user = msg['msg']['agent']
            project = msg['msg']['instance']['project_name']
            if 'ticket.delete' in msg['topic']:
                tmpl = self._(
                    "{user} straight-up deleted a ticket "
                    "on the {project} trac instance")
            elif 'ticket.new' in msg['topic']:
                tmpl = self._(
                    "{user} opened a new ticket "
                    "on the {project} trac instance")
            elif 'ticket.update' in msg['topic']:
                tmpl = self._(
                    "{user} updated a ticket "
                    "on the {project} trac instance")
                status = msg['msg']['ticket']['status']
                old_values = msg['msg']['old_values']
                if status == 'reopened' and 'status' in old_values:
                    tmpl = self._(
                        "{user} reopened a ticket "
                        "on the {project} trac instance")
                elif status == 'closed' and 'status' in old_values:
                    resolution = msg['msg']['ticket']['resolution']
                    tmpl = self._(
                        "{user} closed a ticket "
                        "on the {project} trac instance "
                        "as '{resolution}'")

            return tmpl.format(
                user=user,
                project=project,
                status=status,
                resolution=resolution,
            )
        elif 'git.receive' in msg['topic']:
            user = msg['msg']['commit']['username']
            project = repo_name(msg)

            tmpl = self._(
                "{user} pushed some commits to the '{project}' "
                "fedorahosted git repository")

            return tmpl.format(user=user, project=project)

    def secondary_icon(self, msg, **config):
        user = None

        try:
            user = msg['msg']['agent']
        except KeyError:
            pass

        try:
            user = msg['msg']['commit']['username']
        except KeyError:
            pass

        if not user:
            return ""

        return gravatar_url(username=user)

    def usernames(self, msg, **config):
        users = set()

        try:
            users.add(msg['msg']['agent'])
        except KeyError:
            pass

        try:
            user = msg['msg']['page']['author'].strip()
            if user:
                users.add(user)
        except KeyError:
            pass

        try:
            user = msg['msg']['ticket']['owner'].strip()
            if user:
                users.add(user)
        except KeyError:
            pass

        try:
            user = msg['msg']['ticket']['reporter'].strip()
            if user:
                users.add(user)
        except KeyError:
            pass

        try:
            for user in msg['msg']['ticket']['cc'].split(','):
                user = user.strip()
                if user:
                    users.add(user)
        except KeyError:
            pass

        try:
            user = msg['msg']['commit']['username'].strip()
            if user:
                users.add(user)
        except KeyError:
            pass

        return users

    def objects(self, msg, **config):
        if 'trac.wiki.page.rename' in msg['topic']:
            project = msg['msg']['instance']['project_name']
            name = msg['msg']['page']['name']
            old_name = msg['msg']['old_name']
            return set([
                '/'.join([project, 'wiki', name]),
                '/'.join([project, 'wiki', old_name]),
            ])
        elif 'trac.wiki.' in msg['topic']:
            project = msg['msg']['instance']['project_name']
            name = msg['msg']['page']['name']
            return set(['/'.join([project, 'wiki', name])])
        elif 'trac.ticket.' in msg['topic']:
            project = msg['msg']['instance']['project_name']
            id = msg['msg']['ticket']['id']
            return set(['/'.join([project, 'ticket', str(id)])])
        elif '.git.receive' in msg['topic']:
            files = msg['msg']['commit']['stats']['files'].keys()
            project = repo_name(msg)
            return set([
                project + '/git/' + filename
                for filename in files
            ])

    def link(self, msg, **config):
        if 'trac.wiki.' in msg['topic']:
            base_url = msg['msg']['instance']['base_url'].strip('/')
            name = msg['msg']['page']['name']
            url = '/'.join([base_url, 'wiki', name])

            version = msg['msg']['page']['version']
            versioned = version and all([
                '.rename' not in msg['topic'],
                '.new' not in msg['topic'],
            ])
            if versioned:
                url += "?action=diff&version=%i" % version

            return url
        elif 'trac.ticket.' in msg['topic']:
            base_url = msg['msg']['instance']['base_url'].strip('/')
            id = msg['msg']['ticket']['id']
            url = '/'.join([base_url, 'ticket', str(id)])
            return url
        elif '.git.receive' in msg['topic']:
            name = repo_name(msg)
            rev = msg['msg']['commit']['rev']
            tmpl = "https://git.fedorahosted.org/cgit/" + \
                "{name}.git/commit/?id={rev}"
            return tmpl.format(name=name, rev=rev)



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

from fasshim import gravatar_url, gravatar_url_from_email, email2fas
from fedmsg_meta_fedora_infrastructure import BaseProcessor

MAX_LEN = 40


def comma_join(fields, oxford=True):
    """ Join together words. """

    def fmt(field):
        return "'%s'" % field

    if not fields:
        return "nothing"
    elif len(fields) == 1:
        return fmt(fields[0])
    elif len(fields) == 2:
        return " and ".join([fmt(f) for f in fields])
    else:
        result = ", ".join([fmt(f) for f in fields[:-1]])
        if oxford:
            result += ","
        result += " and %s" % fmt(fields[-1])
        return result


class BugzillaProcessor(BaseProcessor):
    __name__ = "bugzilla"
    __description__ = "Red Hat Bugzilla"
    __link__ = "https://bugzilla.redhat.com"
    __icon__ = "https://apps.fedoraproject.org/img/icons/bugzilla.png"
    __docs__ = "https://bugzilla.redhat.com"
    __obj__ = "Bug Updates"

    def _email_to_fas(self, email, **config):
        user = email2fas(email, **config)

        if '@' in user:
            is_fas = False
        else:
            is_fas = True

        return user, is_fas

    def _get_user(self, msg, **config):
        email = msg['msg']['event'].get('who')
        if not email:
            email = msg['msg']['bug'].get('creator')

        user, is_fas = self._email_to_fas(email, **config)
        return user, is_fas

    def link(self, msg, **config):
        return msg['msg']['bug']['weburl']

    def subtitle(self, msg, **config):
        user, is_fas = self._get_user(msg, **config)
        idx = msg['msg'].get('bug', {}).get('id')
        title = msg['msg'].get('bug', {}).get('summary')

        if len(title) > MAX_LEN:
            title = title[:MAX_LEN] + "..."

        if 'bug.update' in msg['topic']:
            fields = [d['field_name'] for d in msg['msg']['event']['changes']]
            fields = comma_join(fields)
            tmpl = self._("{user} updated {fields} on RHBZ#{idx} '{title}'")
            return tmpl.format(user=user, fields=fields, idx=idx, title=title)
        elif 'bug.new' in msg['topic']:
            tmpl = self._("{user} filed a new bug RHBZ#{idx} '{title}'")
            return tmpl.format(user=user, idx=idx, title=title)

    def secondary_icon(self, msg, **config):
        user, is_fas = self._get_user(msg, **config)
        if is_fas:
            return gravatar_url(user)
        else:
            return gravatar_url_from_email(user)

    def _gather_emails(self, msg):
        users = set()
        msg = msg['msg']
        bug = msg['bug']

        users.add(msg['event'].get('who'))
        users.add(bug.get('creator'))
        users.add(bug.get('assigned_to'))

        for email in bug['cc']:
            users.add(email)

        # Strip anything that made it in erroneously
        for user in list(users):
            if not user:
                users.remove(user)
                continue
            if user.endswith('lists.fedoraproject.org'):
                users.remove(user)
                continue

        return users

    def usernames(self, msg, **config):
        emails = self._gather_emails(msg)
        users = set()
        for email in emails:
            user, is_fas = self._email_to_fas(email, **config)
            if is_fas:
                users.add(user)
        return users

    def objects(self, msg, **config):
        return set([
            '/'.join([
                msg['msg']['bug']['product'],
                msg['msg']['bug']['component'],
                str(msg['msg']['bug']['id']),
            ])
        ])



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
from fedmsg_meta_fedora_infrastructure import BaseProcessor
from fasshim import gravatar_url


class TaggerProcessor(BaseProcessor):
    __name__ = "fedoratagger"
    __description__ = "the Fedora package labeller/tagger"
    __link__ = "https://apps.fedoraproject.org/tagger"
    __docs__ = "https://github.com/ralphbean/fedora-tagger"
    __icon__ = "https://apps.fedoraproject.org/img/icons/tagger.png"
    __obj__ = "Package Tag Votes"

    def link(self, msg, **config):
        vote = msg.get('msg', {}).get('vote', {})
        pack = vote.get('tag', {}).get('package', None)

        # Backwards compat with old tagger messages
        if hasattr(pack, 'keys'):
            pack = pack.keys()[0]

        if not pack and 'rating' in msg['msg']:
            pack = msg['msg']['rating']['package']['name']

        if 'package' in msg['msg']:
            pack = msg['msg']['package']['name']

        if pack:
            return "https://apps.fedoraproject.org/tagger/" + pack
        else:
            return ""

    def subtitle(self, msg, **config):
        if 'fedoratagger.tag.update' in msg['topic']:
            user = msg['msg']['vote']['user']['username']
            tag = msg['msg']['vote']['tag']['tag']
            package = msg['msg']['vote']['tag']['package']

            # Backwards compat with old tagger messages
            if hasattr(package, 'keys'):
                package = package.keys()[0]

            if msg['msg']['vote']['like']:
                verb = "up"
            else:
                verb = "down"

            tmpl = self._('{user} {verb}voted "{tag}" on {package}')
            return tmpl.format(user=user, tag=tag, verb=verb, package=package)
        elif 'fedoratagger.rating.update' in msg['topic']:
            user = msg['msg']['rating']['user']
            if user.get('anonymous', False):
                user = self._('An anonymous user')
            else:
                user = user['username']
            package = msg['msg']['rating']['package']['name']
            rating = msg['msg']['rating']['rating']
            tmpl = self._('{user} gave {package} a rating of {rating}')
            return tmpl.format(user=user, package=package, rating=rating)
        elif 'fedoratagger.tag.create' in msg['topic']:
            user = msg['msg']['vote']['user']['username']
            tag = msg['msg']['vote']['tag']['tag']
            package = msg['msg']['vote']['tag']['package']

            # Backwards compat with old tagger messages
            if hasattr(package, 'keys'):
                package = package.keys()[0]

            tmpl = self._('{user} added tag "{tag}" to {package}')
            return tmpl.format(user=user, tag=tag, package=package)
        elif 'fedoratagger.user.rank.update' in msg['topic']:
            user = msg['msg']['user']['username']
            rank = msg['msg']['user']['rank']
            tmpl = self._("{user}'s rank changed to {rank}")
            return tmpl.format(user=user, rank=rank)
        elif 'fedoratagger.usage.toggle' in msg['topic']:
            user = msg['msg']['user']
            if user.get('anonymous', False):
                user = self._('An anonymous user')
            else:
                user = user['username']
            package = msg['msg']['package']['name']
            if msg['msg']['usage']:
                tmpl = self._(
                    "{user} declared that they use {package}")
            else:
                tmpl = self._(
                    "{user} declared that they no longer use {package}")
            return tmpl.format(user=user, package=package)
        else:
            raise NotImplementedError("%r" % msg)

    def secondary_icon(self, msg, **config):
        usernames = self.usernames(msg, **config)
        if usernames:
            return gravatar_url(list(usernames)[0])
        else:
            return self.__icon__

    def usernames(self, msg, **config):
        user = 'anonymous'
        if 'fedoratagger.tag.' in msg['topic']:
            user = msg['msg']['vote']['user']['username']
        elif 'fedoratagger.user.rank.update' in msg['topic']:
            user = msg['msg']['user']['username']
        elif 'fedoratagger.rating.update' in msg['topic']:
            user = msg['msg']['rating']['user']['username']
        elif 'fedoratagger.usage.toggle' in msg['topic']:
            user = msg['msg']['user']['username']

        if user is 'anonymous':
            return set()
        else:
            return set([user])

    def packages(self, msg, **config):
        if 'fedoratagger.tag.' in msg['topic']:
            package = msg['msg']['vote']['tag']['package']

            # Backwards compat with old tagger messages
            if hasattr(package, 'keys'):
                package = package.keys()[0]

            return set([package])
        elif 'fedoratagger.rating.update' in msg['topic']:
            package = msg['msg']['rating']['package']['name']
            return set([package])
        elif 'fedoratagger.usage.toggle' in msg['topic']:
            package = msg['msg']['package']['name']
            return set([package])

        return set()

    def objects(self, msg, **config):
        packages = self.packages(msg, **config)
        objs = set(['packages/' + p for p in packages])
        if 'vote' in msg['msg']:
            objs.add('labels/' + msg['msg']['vote']['tag']['tag'])
        return objs



# This file is part of fedmsg.
# Copyright (C) 2012-2014 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>

from fasshim import gravatar_url
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class GithubProcessor(BaseProcessor):
    __name__ = "github"
    __description__ = "Fedora-related Github Repos"
    __link__ = "https://github.com"
    __icon__ = "https://apps.fedoraproject.org/img/icons/github.png"
    __docs__ = "http://developer.github.com/webhooks/#events"
    __obj__ = "Github Events"

    def _get_user(self, msg):
        if 'pusher' in msg['msg']:
            pusher = msg['msg']['pusher']['name']
            return msg['msg']['fas_usernames'].get(pusher, pusher)
        if 'sender' in msg['msg']:
            sender = msg['msg']['sender']['login']
            return msg['msg']['fas_usernames'].get(sender, sender)
        if 'forkee' in msg['msg']:
            forkee = msg['msg']['forkee']['owner']['login']
            return msg['msg']['fas_usernames'].get(forkee, forkee)
        return None

    def _get_repo(self, msg):
        try:
            repo = msg['msg']['repository']
            if 'login' in repo['owner']:
                return repo['owner']['login'] + '/' + repo['name']
            else:
                return repo['owner']['name'] + '/' + repo['name']

        except KeyError:
            return None

    def link(self, msg, **config):
        if 'compare' in msg['msg']:
            return msg['msg']['compare']
        if 'pull_request' in msg['msg']:
            return msg['msg']['pull_request']['html_url']
        if 'comment' in msg['msg']:
            return msg['msg']['comment']['html_url']
        if 'issue' in msg['msg']:
            return msg['msg']['issue']['html_url']
        if 'forkee' in msg['msg']:
            return msg['msg']['forkee']['html_url']
        if 'repository' in msg['msg']:
            return msg['msg']['repository']['html_url']

    def subtitle(self, msg, **config):
        user = self._get_user(msg)
        repo = self._get_repo(msg)
        if 'github.webhook' in msg['topic']:
            return self._('A new github repository has been added to fedmsg')
        elif 'github.push' in msg['topic']:
            n = len(msg['msg']['commits'])
            tmpl = self._('{user} pushed {n} commit(s) to {repo}')
            return tmpl.format(user=user, repo=repo, n=n)
        elif 'github.pull_request.' in msg['topic']:
            action = msg['msg']['action']
            n = msg['msg']['number']
            tmpl = self._('{user} {action} pull request #{n} on {repo}')
            return tmpl.format(user=user, action=action, n=n, repo=repo)
        elif 'github.issue.' in msg['topic']:
            if 'comment' in msg['msg']:
                action = 'commented on'
            else:
                action = msg['msg']['action']

            try:
                n = msg['msg']['number']
            except KeyError:
                n = msg['msg']['issue']['number']

            tmpl = self._('{user} {action} issue #{n} on {repo}')
            return tmpl.format(user=user, action=action, n=n, repo=repo)
        elif 'github.create' in msg['topic']:
            typ = msg['msg']['ref_type']
            ref = msg['msg']['ref']
            tmpl = self._('{user} created a new {typ} "{ref}" at {repo}')
            return tmpl.format(user=user, repo=repo, ref=ref, typ=typ)
        elif 'github.fork' in msg['topic']:
            tmpl = self._('{user} forked {repo}')
            return tmpl.format(user=user, repo=repo)
        else:
            pass

    def secondary_icon(self, msg, **config):
        if 'github.webhook' in msg['topic']:
            return self.icon(msg, **config)
        # Otherwise
        return gravatar_url(self._get_user(msg))

    def usernames(self, msg, **config):
        return set(msg['msg']['fas_usernames'].values())

    def objects(self, msg, **config):
        suffix = '.'.join(msg['topic'].split('.')[3:5])
        lookup = {
            'github.push': 'git',
            'github.pull_request': 'pull_request',
            'github.issue': 'issue',
            'github.fork': 'forks',
            'github.create': 'create',
        }

        if suffix not in lookup:
            return set()

        base = lookup[suffix] + '/' + self._get_repo(msg)

        items = []
        if 'commits' in msg['msg']:
            for c in msg['msg']['commits']:
                items = c['added'] + c['removed'] + c['modified']
        elif suffix == 'github.pull_request':
            items = ['%s' % msg['msg']['number']]
        elif suffix == 'github.issue':
            try:
                n = msg['msg']['number']
            except KeyError:
                n = msg['msg']['issue']['number']
            items = ['%s' % n]
        elif suffix == 'github.fork':
            items = [self._get_user(msg)]
        elif suffix == 'github.create':
            items = ['/'.join([msg['msg']['ref_type'], msg['msg']['ref']])]

        return set([base + '/' + item for item in items])



#-*- coding: utf-8 -*-

import fedora.client
import fedmsg.meta.base


class BaseProcessor(fedmsg.meta.base.BaseProcessor):
    FAS = fedora.client.AccountSystem()

    def emails(self, msg, **config):
        usernames = self.usernames(msg, **config)
        emails = [name + "@fedoraproject.org" for name in usernames]
        return dict(zip(emails, usernames))

    def avatars(self, msg, **config):
        usernames = self.usernames(msg, **config)
        kwargs = dict(lookup_email=False)
        lookup = lambda x: self.FAS.avatar_url(x, **kwargs)
        urls = map(lookup, usernames)
        return dict(zip(usernames, urls))



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for cnucnuweb messages """

import unittest

from fedmsg_meta_fedora_infrastructure.tests.base import Base

from common import add_doc


class TestNewDistro(Base):
    """ These messages are published when a new Linux distribution is added
    to the database of `cnucnuweb
    <https://github.com/fedora-infra/cnucnuweb>`_.
    """
    expected_title = "cnucnuweb.distro.add"
    expected_subti = 'ralph added the distro named "Fedora" to cnucnuweb'
    expected_link = "https://apps.fedoraproject.org/cnucnu/distros"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['distros/Fedora'])
    msg = {
        u'i': 1,
        u'msg': {
            u'distro': {
                u'name': u'Fedora',
            },
            u'message': {
                u'agent': u'rbean@redhat.com',
                u'distro': u'Debian',
            },
            u'project': None,
        },
        "i": 2,
        "timestamp": 1386821177,
        "msg_id": "2013-d814724a-8ca3-4e8d-936a-e4195e93336c",
        "topic": "org.fedoraproject.prod.cnucnuweb.distro.add",
    }


class TestEditDistro(Base):
    """ These messages are published when a Linux distribution's entry is
    edited in the `cnucnuweb <https://github.com/fedora-infra/cnucnuweb>`_
    database.
    """
    expected_title = "cnucnuweb.distro.edit"
    expected_subti = 'ralph changed a distro name from "Fedora" to "FancyHat"'
    expected_link = "https://apps.fedoraproject.org/cnucnu/distros"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['distros/Fedora', 'distros/FancyHat'])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1386821512,
        "msg_id": "2013-f18cfb35-894d-41e6-9fda-6f7b99e7e003",
        "topic": "org.fedoraproject.prod.cnucnuweb.distro.edit",
        "msg": {
            "project": None,
            "message": {
                "new": "FancyHat",
                "old": "Fedora",
                "agent": "rbean@redhat.com"
            },
            "distro": {
                "name": "Fedora"
            }
        }
    }


class TestAddProject(Base):
    """ These messages are published when someone adds a new project to
    `cnucnuweb's <https://github.com/fedora-infra/cnucnuweb>`_ database.
    """
    expected_title = "cnucnuweb.project.add"
    expected_subti = 'ralph added the project "ansi2html" to cnucnuweb'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/ansi2html'])
    msg = {
        "username": "apache",
        "i": 4,
        "timestamp": 1386821688,
        "msg_id": "2013-154429ec-842e-4d7f-acae-8d7434b4cbff",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.add",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386839688.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386839688.0,
                "packages": [],
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "agent": "rbean@redhat.com"
            },
            "distro": None,
        }
    }


class TestAddProjectTried(Base):
    """ These messages are published when someone *tries* to add a new project
    to `cnucnuweb's <https://github.com/fedora-infra/cnucnuweb>`_ database.
    """
    expected_title = "cnucnuweb.project.add.tried"
    expected_subti = 'ralph tried to add the project "ansi2html" to cnucnuweb'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/ansi2html'])
    msg = {
        "username": "apache",
        "i": 4,
        "timestamp": 1386821688,
        "msg_id": "2013-154429ec-842e-4d7f-acae-8d7434b4cbff",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.add.tried",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386839688.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386839688.0,
                "packages": [],
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "agent": "rbean@redhat.com"
            },
            "distro": None,
        }
    }


class TestEditProject(Base):
    """ These messages are published when someone edits the details of a
    project in `cnucnuweb's <https://github.com/fedora-infra/cnucnuweb>`_
    database.
    """
    expected_title = "cnucnuweb.project.edit"
    expected_subti = 'ralph edited the following fields of the "ansi2html"' + \
        ' project: homepage, regex'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/ansi2html'])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1386822064,
        "msg_id": "2013-35ba0f89-762e-4ed2-a686-484a6862beb6",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.edit",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386839688.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386840021.0,
                "packages": [],
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "fields": [
                    "homepage",
                    "regex"
                ],
                "agent": "rbean@redhat.com"
            },
            "distro": None
        }
    }


class TestRemoveProject(Base):
    """ These messages are published when someone *removes* a project from
    `cnucnuweb's <https://github.com/fedora-infra/cnucnuweb>`_ database.
    """
    expected_title = "cnucnuweb.project.remove"
    expected_subti = 'ralph deleted the "ansi2html" project'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['projects/ansi2html'])
    msg = {
        "username": "apache",
        "i": 2,
        "timestamp": 1386822329,
        "msg_id": "2013-738f0d92-221e-4894-a3bc-5bf865199529",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.remove",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386839688.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386840064.0,
                "packages": [],
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "agent": "rbean@redhat.com"
            },
            "distro": None
        }
    }


class TestNewMappingProject(Base):
    """ These messages are published when someone maps an upstream project to a
    package name in a particular distribution (in the `cnucnuweb
    <https://github.com/fedora-infra/cnucnuweb>`_ database...)
    """
    expected_title = "cnucnuweb.project.map.new"
    expected_subti = 'ralph mapped the name of "ansi2html" in Fedora to' + \
        ' "python-ansi2html"'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set(['python-ansi2html'])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'projects/ansi2html',
        'mappings/Fedora/python-ansi2html',
        'distros/Fedora',
    ])
    msg = {
        "username": "apache",
        "i": 1,
        "timestamp": 1386823075,
        "msg_id": "2013-7e860d4e-d9eb-41c6-b693-f0786b45db19",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.map.new",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386838297.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386838297.0,
                "homepage": "ansi2html.com",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "new": "python-ansi2html",
                "agent": "rbean@redhat.com",
                "distro": "Fedora"
            },
            "distro": {
                "name": "Fedora",
            },
        }
    }


class TestUpdatedMappingProject(Base):
    """ These messages are published when someone updates the mapping between
    an upstream project and a package name in a particular distribution (in the
    `cnucnuweb <https://github.com/fedora-infra/cnucnuweb>`_ database...)
    """
    expected_title = "cnucnuweb.project.map.update"
    expected_subti = 'ralph updated the name of "ansi2html" in "Fedora" ' + \
        'from "python-ansi2html" to "python3-ansi2html"'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    expected_secondary_icon = "http://www.gravatar.com/avatar/" + \
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F" + \
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png"
    expected_packages = set([
        'python-ansi2html',
        'python3-ansi2html',
    ])
    expected_usernames = set(['ralph'])
    expected_objects = set([
        'projects/ansi2html',
        'mappings/Fedora/python-ansi2html',
        'mappings/Fedora/python3-ansi2html',
        'distros/Fedora',
    ])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1386824705,
        "msg_id": "2013-319d6a62-827c-458b-971c-6528f6e71dc3",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.map.update",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386842519.0,
                "version": None,
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386842519.0,
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "message": {
                "project": "ansi2html",
                "new": "python3-ansi2html",
                "prev": "python-ansi2html",
                "agent": "rbean@redhat.com",
                "distro": "Fedora"
            },
            "distro": {
                "name": "Fedora"
            }
        }
    }


class TestNewUpstreamVersion(Base):
    """ The purpose of cnucnuweb is to monitor upstream projects and to
    try and detect when they release new tarballs.  *These* messages are the
    ones that get published when a tarball is found that is newer than the
    one last seen in the `cnucnuweb
    <https://github.com/fedora-infra/cnucnuweb>`_ database.
    """
    expected_title = "cnucnuweb.project.version.update"
    expected_subti = 'A new version of "ansi2html" has been detected:  ' + \
        '"0.1.4" in advance of "0.1.3"'
    expected_link = "https://apps.fedoraproject.org/cnucnu/project/ansi2html/"
    expected_icon = "http://todo.com/image.png"
    #expected_secondary_icon = None
    expected_packages = set([
        'python-ansi2html',
    ])
    expected_usernames = set([])
    expected_objects = set([
        'projects/ansi2html',
    ])
    msg = {
        "username": "apache",
        "i": 3,
        "timestamp": 1386824705,
        "msg_id": "2013-319d6a62-827c-458b-971c-6528f6e71dc3",
        "topic": "org.fedoraproject.prod.cnucnuweb.project.version.update",
        "msg": {
            "project": {
                "regex": "DEFAULT:ansi2html",
                "logs": None,
                "created_on": 1386842519.0,
                "version": "0.1.4",
                "version_url": "PYPI-DEFAULT:ansi2html",
                "updated_on": 1386842519.0,
                "homepage": "https://github.com/ralphbean/ansi2html",
                "name": "ansi2html"
            },
            "upstream_version": "0.1.4",
            "old_version": "0.1.3",
            "strange": False,
        }
    }


add_doc(locals())

if __name__ == '__main__':
    unittest.main()



import urllib
import socket
from hashlib import md5

_valid_gravatar_sizes = (32, 64, 140)

_fas_cache = {}

import logging
log = logging.getLogger("moksha.hub")


def gravatar_url(username, size=64, default=None):
    try:
        import fedora.client
        system = fedora.client.AccountSystem()
        return system.gravatar_url(
            username, size, default, lookup_email=False)
    except Exception:
        email = username + "@fedoraproject.org"
        return gravatar_url_from_email(email, size, default)


def gravatar_url_from_email(email, size=64, default=None):
    """
    Our own implementation since fas doesn't support this nicely yet.
    """

    if size not in _valid_gravatar_sizes:
        raise ValueError(b_(
            'Size %(size)i disallowed.  Must be in %(valid_sizes)r') % {
                'size': size, 'valid_sizes': _valid_gravatar_sizes})

    if not default:
        default = ("http://fedoraproject.org/static/images/"
                   "fedora_infinity_%ix%i.png" % (size, size))

    return _kernel(email, size, default, service='gravatar')


def _kernel(email, size, default, service='gravatar'):
    """ Copy-and-paste of some code from python-fedora. """

    if service == 'libravatar':
        import libravatar
        return libravatar.libravatar_url(
            email=email,
            size=size,
            default=default,
        )
    else:
        query_string = urllib.urlencode({
            's': size,
            'd': default,
        })

        hash = md5(email).hexdigest()

        return "http://www.gravatar.com/avatar/%s?%s" % (hash, query_string)


def make_fas_cache(**config):
    global _fas_cache
    if _fas_cache:
        return _fas_cache

    log.warn("No previous fas cache found.  Looking to rebuild.")

    try:
        import fedora.client
        import fedora.client.fas2
    except ImportError:
        log.warn("No python-fedora installed.  Not caching fas.")
        return {}

    if not 'fas_credentials' in config:
        log.warn("No fas_credentials found.  Not caching fas.")
        return {}

    creds = config['fas_credentials']

    fasclient = fedora.client.fas2.AccountSystem(
        username=creds['username'],
        password=creds['password'],
    )

    timeout = socket.getdefaulttimeout()
    socket.setdefaulttimeout(600)
    try:
        log.info("Downloading FAS cache")
        request = fasclient.send_request('/user/list',
                                         req_params={'search': '*'},
                                         auth=True)
    except fedora.client.ServerError as e:
        log.error("Failed to download fas cache %r" % e)
        return {}
    finally:
        socket.setdefaulttimeout(timeout)

    log.info("Caching necessary user data")
    for user in request['people']:
        nick = user['ircnick']
        if nick:
            _fas_cache[nick] = user['username']

        email = user['email']
        if email:
            _fas_cache[email] = user['username']

    del request
    del fasclient
    del fedora.client.fas2

    return _fas_cache


def nick2fas(nickname, **config):
    fas_cache = make_fas_cache(**config)
    return fas_cache.get(nickname, nickname)


def email2fas(email, **config):
    fas_cache = make_fas_cache(**config)
    return fas_cache.get(email, email)



# This file is part of fedmsg.
# Copyright (C) 2012 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#
""" Tests for fedocal/calendar messages """

import unittest
import datetime

from fedmsg.tests.test_meta import Base

from common import add_doc


class TestCalendarClear(Base):
    """ These messages are published when someone clears a calendar of all
    its meeting in `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.calendar.clear"
    expected_subti = 'ralph cleared the "awesome" calendar of all its ' \
        'meetings'
    expected_link = "https://apps.fedoraproject.org/calendar/awesome/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/clear'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.dev.fedocal.calendar.clear",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "agent": "ralph"
        }
    }


class TestCalendarCreate(Base):
    """ These messages are published when someone creates a whole calendar from
    `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.calendar.new"
    expected_subti = 'ralph created a whole new "awesome" calendar'
    expected_link = "https://apps.fedoraproject.org/calendar/awesome/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/new'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.dev.fedocal.calendar.new",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "agent": "ralph"
        }
    }


class TestCalendarDelete(Base):
    """ These messages are published when someone deletes a whole calendar from
    `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.calendar.delete"
    expected_subti = 'ralph deleted the "awesome" calendar'
    expected_link = "https://apps.fedoraproject.org/calendar/awesome/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/delete'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.dev.fedocal.calendar.delete",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "agent": "ralph"
        }
    }


class TestCalendarUpdate(Base):
    """ These messages are published when someone updates a whole calendar from
    `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.calendar.update"
    expected_subti = 'ralph updated the "awesome" calendar'
    expected_link = "https://apps.fedoraproject.org/calendar/awesome/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/update'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.dev.fedocal.calendar.update",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "agent": "ralph"
        }
    }


class TestCalendarUpload(Base):
    """ These messages are published when someone uploads an iCalendar file
    into a calendar of `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.calendar.upload"
    expected_subti = 'ralph uploaded an iCalendar file into the calendar ' \
        '"awesome"'
    expected_link = "https://apps.fedoraproject.org/calendar/awesome/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/upload'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638157.759283,
        "msg_id": "2013-96f9ca0e-c7c6-43f0-9de7-7a268c7f1cef",
        "topic": "org.fedoraproject.dev.fedocal.calendar.upload",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "agent": "ralph"
        }
    }


class TestMeetingCreate(Base):
    """ These messages are published when someone creates a meeting from
    `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.meeting.new"
    expected_subti = ('ralph created a "wat" meeting in '
                      'the "awesome" calendar')
    expected_link = "https://apps.fedoraproject.org/calendar/meeting/42/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/meetings/wat/new'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638613.767245,
        "msg_id": "2013-8d60e263-5c5f-40bb-86e0-241dc3965ba4",
        "topic": "org.fedoraproject.dev.fedocal.meeting.new",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "meeting": {
                "meeting_time_start": "12:00:00",
                "meeting_name": "wat",
                "meeting_id": 42,
                "meeting_time_stop": "12:00:00",
                "calendar_name": "awesome",
                "meeting_date_end": "2013-09-21",
                "meeting_timezone": "UTC",
                "meeting_manager": "ralph,",
                "meeting_date": "2013-09-20",
                "meeting_information": "awesome",
                "meeting_region": None
            },
            "agent": "ralph"
        }
    }


class TestMeetingUpdate(Base):
    """ These messages are published when someone updates a meeting from
    `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.meeting.update"
    expected_subti = ('ralph updated the "wat" meeting from '
                      'the "awesome" calendar')
    expected_link = "https://apps.fedoraproject.org/calendar/meeting/42/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/meetings/wat/update'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638613.767245,
        "msg_id": "2013-8d60e263-5c5f-40bb-86e0-241dc3965ba4",
        "topic": "org.fedoraproject.dev.fedocal.meeting.update",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "meeting": {
                "meeting_time_start": "12:00:00",
                "meeting_name": "wat",
                "meeting_id": 42,
                "meeting_time_stop": "12:00:00",
                "calendar_name": "awesome",
                "meeting_date_end": "2013-09-21",
                "meeting_timezone": "UTC",
                "meeting_manager": "ralph,",
                "meeting_date": "2013-09-20",
                "meeting_information": "awesome",
                "meeting_region": None
            },
            "agent": "ralph"
        }
    }


class TestMeetingDelete(Base):
    """ These messages are published when someone deletes a meeting from
    `fedocal <https://apps.fedoraproject.org/calendar>`_.
    """

    expected_title = "fedocal.meeting.delete"
    expected_subti = ('ralph deleted the "wat" meeting from '
                      'the "awesome" calendar')
    expected_link = "https://apps.fedoraproject.org/calendar/meeting/42/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_secondary_icon = (
        "http://www.gravatar.com/avatar/"
        "2f933f4364baaabd2d3ab8f0664faef2?s=64&d=http%3A%2F%2F"
        "fedoraproject.org%2Fstatic%2Fimages%2Ffedora_infinity_64x64.png")
    expected_packages = set([])
    expected_usernames = set(['ralph'])
    expected_objects = set(['awesome/meetings/wat/delete'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638613.767245,
        "msg_id": "2013-8d60e263-5c5f-40bb-86e0-241dc3965ba4",
        "topic": "org.fedoraproject.dev.fedocal.meeting.delete",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "meeting": {
                "meeting_time_start": "12:00:00",
                "meeting_name": "wat",
                "meeting_id": 42,
                "meeting_time_stop": "12:00:00",
                "calendar_name": "awesome",
                "meeting_date_end": "2013-09-21",
                "meeting_timezone": "UTC",
                "meeting_manager": "ralph,",
                "meeting_date": "2013-09-20",
                "meeting_information": "awesome",
                "meeting_region": None
            },
            "agent": "ralph"
        }
    }


now = datetime.datetime.utcnow() + datetime.timedelta(hours=1)


class TestMeetingReminder(Base):
    """ These messages are published by a cronjob when time gets close to
    certain meetings scheduled in the `fedocal
    <https://apps.fedoraproject.org/calendar>`_ calendaring system.
    """

    expected_title = "fedocal.meeting.reminder"
    expected_subti = ('Friendly reminder!  The "wat" meeting from the '
                      '"awesome" calendar starts in 59 minutes')
    expected_link = "https://apps.fedoraproject.org/calendar/meeting/42/"
    expected_icon = ("https://apps.fedoraproject.org/calendar/"
                     "static/calendar.png")
    expected_packages = set([])
    expected_usernames = set([])
    expected_objects = set(['awesome/meetings/wat/reminder'])
    msg = {
        "username": "threebean",
        "i": 1,
        "timestamp": 1379638613.767245,
        "msg_id": "2013-8d60e263-5c5f-40bb-86e0-241dc3965ba4",
        "topic": "org.fedoraproject.dev.fedocal.meeting.reminder",
        "msg": {
            "calendar": {
                "calendar_editor_group": "sysadmin-main",
                "calendar_name": "awesome",
                "calendar_description": "cool deal",
                "calendar_admin_group": "sysadmin-badges",
                "calendar_contact": "ralph@fedoraproject.org",
                "calendar_status": "Enabled"
            },
            "meeting": {
                "meeting_time_start": now.time().strftime("%H:%M:%S"),
                "meeting_name": "wat",
                "meeting_id": 42,
                "meeting_time_stop": "12:00:00",
                "calendar_name": "awesome",
                "meeting_date_end": "2013-09-21",
                "meeting_timezone": "UTC",
                "meeting_manager": "ralph,",
                "meeting_date": now.date().isoformat(),
                "meeting_information": "awesome",
                "meeting_region": None
            },
        }
    }


add_doc(locals())


if __name__ == '__main__':
    unittest.main()



# This file is part of fedmsg.
# Copyright (C) 2012, 2013 Red Hat, Inc.
#
# fedmsg is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# fedmsg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with fedmsg; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
#
# Authors:  Ralph Bean <rbean@redhat.com>
#           Luke Macken <lmacken@redhat.com>

from fasshim import gravatar_url
from fedmsg_meta_fedora_infrastructure import BaseProcessor


class NuancierProcessor(BaseProcessor):
    __name__ = "nuancier"
    __description__ = "the supplemental wallpaper voting system"
    __link__ = "https://apps.fedoraproject.org/nuancier/"
    __docs__ = "https://github.com/fedora-infra/nuancier-lite"
    __obj__ = "Wallpaper Elections"

    def link(self, msg, **config):

        if 'original_url' in msg['msg'].get('candidate', {}):
            return msg['msg']['candidate']['original_url']

        kind = msg['topic'].split('.')[4]
        item = msg['msg']['election']['id']
        return "https://apps.fedoraproject.org/nuancier/%s/%s" % (kind, item)

    def subtitle(self, msg, **config):
        kwargs = dict(
            agent=msg['msg']['agent'],
            name=msg['msg']['election']['name'],
        )
        if 'election.update' in msg['topic']:
            tmpl = self._(
                '{agent} changed the following details '
                'on the "{name}" election: {details}')
            kwargs['details'] = ', '.join(msg['msg']['updated'])
        elif 'election.new' in msg['topic']:
            tmpl = self._(
                '{agent} created a new election "{name}"')
        elif 'candidate.new' in msg['topic']:
            tmpl = self._(
                '{agent} uploaded a new candidate for the '
                '"{name}" wallpaper election')
        elif 'candidate.denied' in msg['topic']:
            tmpl = self._(
                '{agent} denied {author}\'s "{candidate}" submission to the '
                '"{name}" wallpaper election')
            kwargs['author'] = msg['msg']['candidate']['submitter']
            kwargs['candidate'] = msg['msg']['candidate']['name']
        elif 'candidate.approved' in msg['topic']:
            tmpl = self._(
                '{agent} approved {author}\'s "{candidate}" submission to the '
                '"{name}" wallpaper election')
            kwargs['author'] = msg['msg']['candidate']['submitter']
            kwargs['candidate'] = msg['msg']['candidate']['name']
        else:
            tmpl = ''

        return tmpl.format(**kwargs)

    def secondary_icon(self, msg, **config):
        return gravatar_url(msg['msg']['agent'])

    def usernames(self, msg, **config):
        users = [msg['msg']['agent']]

        if 'candidate' in msg['msg']:
            users.append(msg['msg']['candidate']['submitter'])

        return set(users)

    def objects(self, msg, **config):
        kind = msg['topic'].split('.')[-2]
        action = msg['topic'].split('.')[-1]
        year = msg['msg']['election']['year']
        name = msg['msg']['election']['name']
        candidate = msg['msg'].get('candidate', {}).get('name')
        tokens = [year, name, kind, action]
        if candidate:
            tokens.insert(2, candidate)
        return set(['/'.join(map(str, tokens))])
